@@ -19,7 +19,7 @@ Copyright 2003-2004 Peter Beerli, Tallahassee FL
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
- $Id: broyden.c 1800 2011-01-29 13:40:00Z beerli $
+ $Id: broyden.c 2035 2012-05-16 20:03:33Z beerli $
 -------------------------------------------------------*/
 /*! \file broyden.c
 Basic routines for maximum likelihood parameter estimation.
@@ -35,10 +35,6 @@ Basic routines for maximum likelihood parameter estimation.
 #include "sighandler.h"
 #include "migrate_mpi.h"
 
-#ifdef ALTIVEC
-#include "altivec.h"
-#endif
-
 #ifdef DMALLOC_FUNC_CHECK
 #include <dmalloc.h>
 #endif
@@ -533,7 +529,6 @@ destroy_nr (nr_fmt * nr, world_fmt * world)
 
 ///
 /// calculate the likelihood of the parameter estimates
-#ifndef ALTIVEC
 MYREAL
 calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
 {
@@ -555,9 +550,19 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     MYREAL *locallparam;
     MYREAL *localparam;
     MYREAL *sm;
-    MYREAL mu_rate = nr->world->options->mu_rates[locus];
-    MYREAL inv_mu_rate = 1./ mu_rate;
-    MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
+    const MYREAL mu_rate = nr->world->options->mu_rates[locus];
+    const MYREAL inv_mu_rate = 1./ mu_rate;
+    const MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
+    // we calculate the maximum likelihood estiamte of the parameters 
+    // so that all loci have the same Theta*, therefor the inheritance scalar has
+    // to be taken into account,
+    // for nuc, mt, x set to scalars 1.0, 0.25, 0.75, 
+    // we need to maximize 1Theta*, 4Theta*, 1.333Theta*
+    const MYREAL inv_inheritance = nr->world->options->inheritance_scalars[locus];
+    const MYREAL inheritance = 1./ inv_inheritance;
+    const MYREAL tlmu_rate = lmu_rate + log(inheritance);
+    const MYREAL tmu_rate = mu_rate * inheritance;
+    const MYREAL tinv_mu_rate = inv_mu_rate * inheritance;
     MYREAL normaldev;
 #ifdef LONGSUM
 
@@ -596,8 +601,8 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
 
     for (r = 0; r < nr->numpop; r++)
     {
-        locallparam[r] = LOG2 - (locallparam[r] + lmu_rate);
-        localparam[r] = -1. / (localparam[r] * mu_rate); // minus, so that we can loop over all in probG4
+        locallparam[r] = LOG2 - (locallparam[r] + tlmu_rate);
+        localparam[r] = -1. / (localparam[r] * tmu_rate); // minus, so that we can loop over all in probG4
         msta = nr->mstart[r];
         msto = nr->mend[r];
         for (j = msta; j < msto; j++)
@@ -681,121 +686,6 @@ calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
     myfree(locallparam);
     return nr->llike;
 }
-#else /*ALTIVEC*/
-
-MYREAL
-calc_locus_like (nr_fmt * nr, MYREAL *param, MYREAL *lparam, long locus)
-{
-    long g, r, j, copies;
-    const long numpop = nr->world->numpop;
-    const long numpop2 = nr->world->numpop2;
-    int msta, msto;
-    MYREAL gsum = 0;
-    MYREAL ***apg0;
-    MYREAL apgmax = -MYREAL_MAX;
-    //  MYREAL ***apgall = nr->apg;
-    MYREAL *apg;
-    MYREAL *apg0r;
-    timearchive_fmt tyme;
-    tarchive_fmt *tl;
-    MYREAL *geo = nr->world->data->geo;
-    MYREAL *lgeo = nr->world->data->lgeo;
-    MYREAL *locallparam;
-    MYREAL *localparam;
-    MYREAL *sm;
-    MYREAL mu_rate = nr->world->options->mu_rates[locus];
-    MYREAL inv_mu_rate = 1./ mu_rate;
-    MYREAL lmu_rate = nr->world->options->lmu_rates[locus];
-    MYREAL normaldev;
-    long size4 = numpop2 + numpop + numpop;
-    long size;
-    FloatVec *vlocal;
-    FloatVec *vdata;
-    size4 -= (size4 % 4) - 4;
-    size = size4 * QUARTER;
-    vlocal = (FloatVec *) mycalloc (MAX(16,size), sizeof(FloatVec));
-    vdata = (FloatVec *) mycalloc (MAX(16,size), sizeof(FloatVec));
-    locallparam = (MYREAL *) mycalloc ((size4), sizeof (MYREAL));
-    localparam = locallparam + numpop2;
-
-    sm = localparam + numpop;
-    memcpy (localparam, param, sizeof (MYREAL) * numpop);
-    memcpy (locallparam, lparam, sizeof (MYREAL) * numpop2);
-
-    nr->PGC[locus] = 0.0;
-    apg0 = nr->apg0;
-
-    for (r = 0; r < nr->numpop; r++)
-    {
-        locallparam[r] = LOG2 - (locallparam[r] + lmu_rate);
-        localparam[r] = -1. / (localparam[r] * mu_rate); // minus, so that we can loop over all in probG4
-        msta = nr->mstart[r];
-        msto = nr->mend[r];
-        for (j = msta; j < msto; j++)
-        {
-            sm[r] -= geo[j] * param[j] * inv_mu_rate; //minus, so that we can loop over all in probG4
-            locallparam[j] += lgeo[j] - lmu_rate;
-        }
-    }
-    gsum = 0;
-    load_float_floatvec(vlocal, locallparam,numpop2+numpop+numpop);
-    for (r = nr->repstart; r < nr->repstop; r++)
-    {
-        apg = nr->apg[r][locus];
-        apg0r = apg0[r][locus];
-        tyme = nr->atl[r][locus];
-        normaldev =  (*normal_func)(param, tyme.param0, numpop2);
-        //first element
-        tl = &(tyme.tl[0]);
-        copies = tl->copies - 1;
-        gsum += copies;
-        if (copies > 0)
-        {
-//            load_float_floatvec(vdata, tl->data,numpop2+numpop+numpop);
-            apg[0] = tl->lcopies + vdot_product (vlocal, tl->vdata, size) - apg0r[0] + normaldev;
-        }
-        else
-            apg[0] = -MYREAL_MAX;
-        if (apgmax < apg[0])
-            apgmax = apg[0];
-        //other elements
-        for (g = 1; g < tyme.T; g++)
-        {
-            tl = &(tyme.tl[g]);
-            gsum += tl->copies;
-            load_float_floatvec(vdata, tl->data,numpop2+numpop+numpop);
-            apg[g] = tl->lcopies + vdot_product (vlocal,vdata, size) - apg0r[g] + normaldev;
-            if (apg[g] > apgmax)
-                apgmax = apg[g];
-        }
-    }    // end replicates
-    for (r = nr->repstart; r < nr->repstop; r++)
-    {
-        apg = nr->apg[r][locus];
-        apg0r = apg0[r][locus];
-        tyme = nr->atl[r][locus];
-        // first element
-        tl = &(tyme.tl[0]);
-        // first element
-        apg[0] -= apgmax;
-        nr->PGC[locus] += EXP (apg[0]);
-        // all other elements
-        for (g = 1; g < tyme.T; g++)
-        {
-            apg[g] -= apgmax;
-            if (apg[g] > -40.)
-                nr->PGC[locus] += EXP (apg[g]);
-        }
-    }    // replicates
-    nr->apg_max[locus] = apgmax;
-    nr->llike = apgmax + LOG (nr->PGC[locus]) - LOG (gsum);
-    myfree(locallparam);
-    myfree(vlocal);
-    myfree(vdata);
-    return nr->llike;
-}
-#endif /*ALTIVEC*/
-
 
 
 ///
@@ -1207,7 +1097,9 @@ gradient (MYREAL *d, nr_fmt * nr, long locus)
     MYREAL *normalgrad;
     MYREAL mu_rate = nr->world->options->mu_rates[locus];
     MYREAL inv_mu_rate = 1. / mu_rate;
-	
+    // takes care of the inheritance scalars for theta
+    const MYREAL inheritance=nr->world->options->inheritance_scalars[locus];
+
     normalgrad = (MYREAL *) mymalloc(nr->numpop2 * sizeof(MYREAL));
     memset (d, 0, sizeof (MYREAL) * nr->numpop2);
     thetas = nr->param;
@@ -1226,7 +1118,7 @@ gradient (MYREAL *d, nr_fmt * nr, long locus)
             expapg = EXP (nr->apg[r][locus][g]);
             for (pop = 0; pop < numpop; pop++)
             {
-                tet = thetas[pop];
+	      tet = thetas[pop]*inheritance; //INHERIT
                 // the following lines are doing this derivative, but have reduced the division operation (speed)
                 // nr->parts[pop] = -tl[g].p[pop] / tet + tl[g].kt[pop] / (tet * tet * mu_rate) - normalgrad[pop];
                 rtet = 1./(tet * tet * mu_rate);
@@ -1314,7 +1206,7 @@ gradient_longsum (MYREAL *d, nr_fmt * nr, long locus)
     long ptime, ptimeold;
     MYREAL timeinterval;
     //interval[0]=intervals[1]=intervals[2]=0;
-    FPRINTF(stdout,"rawrates=%f %f %f @ %f %f %f \n",rawrates[0],rawrates[1],rawrates[2],rawrates[3],rawrates[4],rawrates[5]);
+    FPRINTF(stdout,"CHECK FUNCTION LONGSUM: rawrates=%f %f %f @ %f %f %f \n",rawrates[0],rawrates[1],rawrates[2],rawrates[3],rawrates[4],rawrates[5]);
     treeparts = mycalloc(nr->numpop2+numpop3,sizeof(MYREAL));
     memset (d, 0, sizeof (MYREAL) * nr->partsize);
     thetas = nr->param;
@@ -1608,20 +1500,24 @@ check_symmetric_d (MYREAL *grad, nr_fmt * nr, long nn)
         if (nr->profilenum > 0)
         {
             z = 0;
-            while (nr->indeks[z] != syms[i][0] && z++ < nr->partsize)
-                ;
+            while (z < nr->partsize && nr->indeks[z] != syms[i][0]) 
+                z++;
             zz = 0;
-            while (nr->indeks[zz] != syms[i][1] && zz++ < nr->partsize)
-                ;
+            while (zz < nr->partsize && nr->indeks[zz] != syms[i][1])
+                zz++;
             if (z < nr->partsize)
-                mm = grad[z];
+		mm = grad[z];
             else
-                mm = 0;
+                mm = 0.0;
             if (zz < nr->partsize)
                 mm += grad[zz];
             //else
             //  mm += 0;
-            grad[z] = grad[zz] = mm;
+	    //fprintf(stderr,"@@@@@@@@@@@@@@@ %li %li %f %li\n",z,zz,mm,nr->partsize);
+            if(z<nr->partsize)
+	      grad[z] = mm;
+            if(zz<nr->partsize)
+	      grad[zz] = mm;
         }
         else
         {
