@@ -22,7 +22,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: data.c 1814 2011-03-20 17:21:07Z beerli $
+$Id: data.c 2032 2012-05-11 15:09:38Z beerli $
 
 -----------------------------------
 add this to the microsatellite data, this will allow to use
@@ -87,10 +87,10 @@ long read_ind_seq (FILE * infile, data_fmt * data, option_fmt * options,
 long read_ind_seq_oneliner (FILE * infile, data_fmt * data, option_fmt * options,
 		       long pop, long ind, long baseread);
 
-void read_hapmap (FILE * infile, data_fmt * data, option_fmt * options,
-	     long locus, long pop);
-void read_distance_fromfile (FILE * dfile, long tips, long nmlength,
-                             MYREAL **m);
+void read_hapmap (FILE * infile, data_fmt * data, option_fmt * options, long pop);
+void
+read_hapmap_genotypes (FILE * infile, data_fmt * data, option_fmt * options, long pop);
+void read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m);
 void finish_read_seq (FILE * infile, data_fmt * data, option_fmt * options,
                       long pop, long baseread);
 void print_alleledata (world_fmt * world, data_fmt * data,
@@ -342,6 +342,11 @@ get_new_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *w
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = options->micro_stepnum;
         break;
+    case 'h':
+    case 'n':
+    case 'u':
+        create_alleles (data, options);
+        break;
     default: /*DNA types*/
       for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = 4;
@@ -433,6 +438,11 @@ get_data (FILE * infile, data_fmt * data, option_fmt * options, world_fmt *world
         for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = options->micro_stepnum;
         break;
+    case 'h':
+    case 'n':
+    case 'u':
+        create_alleles (data, options);
+        break;
     default: /*DNA types*/
       for (pop = 0; pop < data->loci; pop++)
             data->maxalleles[pop] = 4;
@@ -592,7 +602,7 @@ init_data_structure3 (data_fmt * data)
 {
     long locus, pop, maxi;
     data->allele =
-        (allele_fmt **) mycalloc (1, sizeof (allele_fmt *) * data->loci);
+        (char ***) mycalloc (1, sizeof (char **) * data->loci);
     data->subloci = (long *) mycalloc(data->loci,sizeof(long));
     for (locus = 0; locus < data->loci; locus++)
     {
@@ -601,7 +611,10 @@ init_data_structure3 (data_fmt * data)
         for (pop = 0; pop < data->numpop; pop++)
             maxi += data->numalleles[pop][locus];
         data->allele[locus] =
-            (allele_fmt *) mycalloc (1, sizeof (allele_fmt) * maxi);
+            (char **) mycalloc (1, sizeof (char*) * maxi);
+        for (pop = 0; pop < maxi; pop++)
+	  data->allele[locus][pop] =
+            (char *) mycalloc (maxi+1, sizeof (char));
     }
     data->maxalleles = (long *) mycalloc (1, sizeof (long) * data->loci);
     data->skiploci =
@@ -768,10 +781,14 @@ read_sites (data_fmt * data, world_fmt *world, option_fmt *options)
       switch(input[0])
 	{
 	case  '#':
-	  if(input[1]=='@')
+	  if(input[1]=='@' || input[1]=='M')
 	    {
+	      if (input[1]=='M')
+		{
+		  warning("You should use #@M if you want to treat the input as fragment lengths in your infile\n");
+		}
 	      // line is microsatellite instruction
-	      if(input[2]=='M')
+	      if(input[2]=='M' || input[1]=='M')
 		{
 		  FGETS2(&input,&allocbufsize,data->infile); // read the line 
 		  locus--;
@@ -1190,9 +1207,17 @@ read_popdata (FILE * infile, data_fmt * data, long pop, option_fmt * options)
 {
     long ind, baseread = 0;
     long locus = 0;
+    char c;
     if(options->datatype=='h')
       {
-	read_hapmap(infile, data, options, locus, pop);
+	//check whether this is genotypeformat or not
+	if ((c=getc(infile))!='r')
+	  {
+	    ungetc(c,infile);
+	    read_hapmap(infile, data, options, pop);
+	  }
+	else
+	  read_hapmap_genotypes(infile, data, options, pop);
       }
     else
       {
@@ -1372,9 +1397,117 @@ read_alleles (FILE * infile, data_fmt * data, long pop, long ind)
     myfree(isave);
 }
 
+///
+/// reads the standard hapmap genotype data file format using this header
+/// rs# alleles chrom pos strand assembly# center protLSID assayLSID panelLSID QCcode individual# ....
 void
-read_hapmap (FILE * infile, data_fmt * data, option_fmt * options,
-              long locus, long pop)
+read_hapmap_genotypes (FILE * infile, data_fmt * data, option_fmt * options, long pop)
+{
+  int error;
+  long ind;
+    char label1;
+    char label2;
+    long label1count;
+    long label2count;
+    long label12count;
+    char *input;
+    long locus;
+    long genomes = number_genomes(options->datatype);
+    input = (char *) mycalloc(LONGLINESIZE,sizeof(char));
+    // read header
+    FGETS(input,LONGLINESIZE,infile);
+    for(locus=0;locus<data->loci;locus++)
+      {
+	//rs# or comment
+	read_word(infile,input);
+	if(input[0]=='#')
+	  {
+	    fprintf(stdout,"%s\n",input);
+	    FGETS(input,LONGLINESIZE,infile);
+	    locus--;
+	    continue;
+	  }
+	// discard rs#
+	// read allele and record
+	read_word(infile,input);
+	label1 = input[0];// format is like this G/C
+	label2 = input[2];
+	// read chromosome and discard
+	read_word(infile,input);
+	// read position
+	read_word(infile,input);
+	data->position[locus] = atol(input);
+	if(options->printdata)
+	  {
+	    fprintf(stdout,"%20li|",data->position[locus]);
+	  }
+        // read strand must be positive but not checked
+	read_word(infile,input);
+	// read assembly# and discard
+	read_word(infile,input);
+	// read center and discard
+	read_word(infile,input);
+	// read protLSID and discard
+	read_word(infile,input);
+	// read assyLSID and discard
+	read_word(infile,input);
+	// read panelLSID and discard
+	read_word(infile,input);
+	// read QCODE and use as dummy
+	read_word(infile,input);
+ 	data->seq[0]->sites[locus]=1;
+	label1count=0;
+	label2count=0;
+	read_word(infile,input);
+	while(strstr(input,"rs")==NULL && !isdigit(input[0]))
+	  {
+	    if (label1 == input[0])
+	      label1count++;
+	    else
+	      label2count++;
+	    if (label2 == input[1])
+	      label2count++;
+	    else
+	      label1count++;
+	    error = read_word(infile,input);
+	    if (error==EOF)
+	      break;
+	  }
+	unread_word(infile,input);
+	label12count = label1count + label2count;
+	if(options->printdata)
+	  {
+	    fprintf(stdout, " freq[%c]=%f freq[%c]=%f\n",label1,
+		    (float) label1count/label12count,label2, (float) label2count/label12count);
+	  }
+	//was messing with the reading, use fraction data->numind[pop][locus] = label12count;
+	if(options->randomsubset > 0 && options->randomsubset < data->numind[pop][locus])
+	  {
+	    data->numalleles[pop][locus] = options->randomsubset * genomes;
+	    data->numind[pop][locus] = options->randomsubset / genomes;
+	  }
+	else
+	  {
+	    data->numalleles[pop][locus] = data->numind[pop][locus] = label12count;
+	  //	  data->numalleles[pop][locus] = data->numind[pop][locus] * genomes;
+	  }
+	label1count = floor( (((double) label1count/label12count * data->numalleles[pop][locus]))+0.5 );
+	for(ind=0;ind < label1count; ind++)
+	  {
+	    data->yy[pop][ind][locus][0][0] = label1;
+	  }
+	for(ind=label1count;ind < data->numalleles[pop][locus]; ind++)
+	  {
+	    data->yy[pop][ind][locus][0][0] = label2;
+	  }
+      }
+    myfree(input);
+}
+
+///
+/// my own hapmap format derived from the frequency data
+void
+read_hapmap (FILE * infile, data_fmt * data, option_fmt * options,  long pop)
 {
   long ind;
     char label1;
@@ -1383,6 +1516,7 @@ read_hapmap (FILE * infile, data_fmt * data, option_fmt * options,
     long label2count;
     long label12count;
     char *input;
+    long locus;
     long genomes = number_genomes(options->datatype);
     input = (char *) mycalloc(LINESIZE,sizeof(char));
     for(locus=0;locus<data->loci;locus++)
@@ -1418,17 +1552,23 @@ read_hapmap (FILE * infile, data_fmt * data, option_fmt * options,
 		    (float) label1count/label12count,label2, (float) label2count/label12count);
 	  }
 	//was messing with the reading, use fraction data->numind[pop][locus] = label12count;
-	//if(options->randomsubset > 0 && options->randomsubset < data->numind[pop][locus])
-	//  data->numalleles[pop][locus] = options->randomsubset * genomes;
-	//else
-	data->numalleles[pop][locus] = data->numind[pop][locus] * genomes;
-	label1count = floor( ((label1count/label12count * data->numalleles[pop][locus]))+0.5 );
-	label12count = data->numalleles[pop][locus] - label1count;
+	if(options->randomsubset > 0 && options->randomsubset < data->numind[pop][locus])
+	  {
+	    data->numalleles[pop][locus] = options->randomsubset * genomes;
+	    data->numind[pop][locus] = options->randomsubset / genomes;
+	  }
+	else
+	  {
+	    data->numalleles[pop][locus] = label12count;
+	    data->numind[pop][locus] = label12count / genomes;
+	  }
+	// data->numind[pop][locus] * genomes;
+	label1count = floor( (((double) label1count/label12count * data->numalleles[pop][locus]))+0.5 );
 	for(ind=0;ind < label1count; ind++)
 	  {
 	    data->yy[pop][ind][locus][0][0] = label1;
 	  }
-	for(ind=label1count;ind < label12count; ind++)
+	for(ind=label1count;ind < data->numalleles[pop][locus]; ind++)
 	  {
 	    data->yy[pop][ind][locus][0][0] = label2;
 	  }
@@ -1581,7 +1721,7 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m)
 {
     char input[SUPERLINESIZE];
     long i, j;
-
+    int retval;
     if (dfile != NULL)
     {
         // assumes that the dfile is in PHYLIP format
@@ -1594,9 +1734,9 @@ read_distance_fromfile (FILE * dfile, long tips, long nmlength, MYREAL **m)
             for (j = 0; j < tips; j++)
             {
 #ifdef USE_MYREAL_FLOAT
-                fscanf (dfile, "%f", &m[i][j]);
+                retval = fscanf (dfile, "%f", &m[i][j]);
 #else
-                fscanf (dfile, "%lf", &m[i][j]);
+                retval = fscanf (dfile, "%lf", &m[i][j]);
 #endif
 		if((i!=j) && (m[i][j] < EPSILON))
 		  {
@@ -1637,7 +1777,7 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, int **uep,
                 uep[i] = (int *) mycalloc (*uepsites, sizeof (int));
                 FGETS (input, nmlength + 1, uepfile); //reads each line
                 for (j = 0; j < *uepsites; ++j)
-                    fscanf (uepfile, "%i", &uep[i][j]);
+                    retval = fscanf (uepfile, "%i", &uep[i][j]);
                 // reads the last \n from the data matrix
                 FGETS (input, LINESIZE, uepfile);
             }
@@ -1650,10 +1790,10 @@ read_uep_fromfile (FILE * uepfile, long tips, long nmlength, int **uep,
                 uep[i + tips] = (int *) mycalloc (*uepsites, sizeof (int));
                 FGETS (input, nmlength + 1, uepfile); //reads each line
                 for (j = 0; j < *uepsites; ++j)
-                    fscanf (uepfile, "%i", &uep[i][j]);
+                    retval = fscanf (uepfile, "%i", &uep[i][j]);
                 // finished reading first allele, no onto the second
                 for (j = 0; j < *uepsites; ++j)
-                    fscanf (uepfile, "%i", &uep[i + tips][j]);
+                    retval = fscanf (uepfile, "%i", &uep[i + tips][j]);
                 // reads the last \n from the data matrix
                 FGETS (input, LINESIZE, uepfile);
             }
@@ -1719,6 +1859,65 @@ long find_missing(data_fmt *data, long pop, long locus)
     return missing;
 }
 
+
+///
+/// Data set was subsampled and used a random sample of size
+void print_random_subset(FILE * file, data_fmt * data, option_fmt *options)
+{
+  long locus;
+  long pop;
+  char *name;
+  long maxnum;
+  long count;
+  long ind;
+  long length;
+  if(options->randomsubset > 0)
+    {
+      fprintf (file, "\nData set was subsampled and used a random sample of size: %li\n\n", options->randomsubset);
+      fprintf (file, "Locus Population Individuals\n");
+      fprintf (file, "----- ---------- ---------------------------------------------------------------------\n");
+      name = (char*) mycalloc(options->nmlength+1,sizeof(char));
+      for (locus=0; locus < data->loci; locus++)
+	{
+	  fprintf(file,"%5li ",locus+1);
+	  for (pop=0; pop < data->numpop; pop++)
+	    {
+	      fprintf(file, "%-10.10s ", data->popnames[pop]);
+	      maxnum = options->randomsubset < data->numind[pop][locus] ? options->randomsubset : data->numind[pop][locus];
+	      count = 18;
+	      for(ind=0;ind<maxnum;ind++)
+		{
+		  //long index = data->shuffled[pop][locus][ind];
+
+		  if(data->indnames[pop][ind][locus][0]=='\0')
+		    memcpy(name,data->indnames[pop][ind][0],sizeof(char)*options->nmlength);
+		  else
+		    memcpy(name,data->indnames[pop][ind][locus],sizeof(char)*options->nmlength);
+		  remove_trailing_blanks(&name);
+		  length = strlen(name);
+		  if (count+length < LINELENGTH)
+		    {
+		      fprintf(file,"%s ",name);
+		      count += length+1;
+		    }
+		  else
+		    {
+		      fprintf(file,"\n");
+		      fprintf(file,"                 ");
+		      fprintf(file,"%s ",name);
+		      count=17+length+1;
+		    }
+		}
+	      if(pop<data->numpop-1)
+		fprintf(file,"\n      ");
+	      else
+		fprintf(file,"\n");
+	    }
+	}
+      myfree(name);
+    }
+}
+
 void
 print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
                     data_fmt * data)
@@ -1767,6 +1966,11 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
     {
       fprintf (file, "[Fragment length is translated to repeats]\n");
     }
+  else
+    {
+      if (dstring[0]=='M')
+	fprintf (file, "[Data was used as repeat-length information]\n");
+    }
 
   fprintf (file, "Number of loci:                         %20li\n\n",
 	   data->loci);
@@ -1781,9 +1985,15 @@ print_data_summary (FILE * file, world_fmt * world, option_fmt * options,
 	  if(locus % 4 == 0)
 	    fprintf(file,"                      ");
 	  fprintf(file,", %f", options->mutationrate_year[locus]); 
-	}
+	} 
       fprintf(file,"\n"); 
     }
+  // 
+  print_random_subset(file,data,options);
+  if(file!=stdout)
+    {
+      pdf_print_random_subset(data,options);
+    }
   for (pop = 0; pop < data->numpop; pop++)
     {
       length = strlen(data->popnames[pop]);
@@ -1879,8 +2089,11 @@ print_data (world_fmt * world, option_fmt * options, data_fmt * data)
 ///
 /// calculate allele frequency spectrum and print
 /// allele population1 .. populationN All
+/// taking into account population labeling and random_subsets
 void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
 {
+  const double one = 1.0;
+  const double two = 2.0;
   long found;
   long locus;
   long a;
@@ -1888,18 +2101,30 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
   long pop1;
   long ind;
   MYREAL **total;
+  MYREAL *grandtotal;
   MYREAL allfreq;
+  MYREAL f;
+  MYREAL homo = 0.0;
+  MYREAL *avghet;
+  MYREAL v;
+  MYREAL avghet1;
+  MYREAL avghetall = 0.0;
   long *maxalleles;
   long *maxallelepop;
   MYREAL ***freq;
   char *thisallele;
   char *thatallele;
+  MYREAL fx;
+  MYREAL general_homo;
   FILE *outfile = world->outfile;
-  if (strchr (SEQUENCETYPES, options->datatype))
+  long loctotal;
+  boolean second = (strchr(ALLELETYPES,options->datatype)!=NULL);
+  if (strchr (DNASEQUENCETYPES, options->datatype))
     return; /*we do not calculate allele frequencies for DNA sequences*/
   // find total number of alleles
   maxalleles = (long *) mycalloc(data->loci, sizeof(long));
   maxallelepop = (long *) mycalloc(data->numpop, sizeof(long));
+  avghet = (MYREAL *) mycalloc(data->numpop, sizeof(MYREAL));
   for (locus = 0; locus < data->loci; locus++)
     {
       maxalleles[locus] = findAllele(data,"\0",locus);
@@ -1910,6 +2135,7 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
     }
 
   // create bins for each population and all
+  grandtotal = (MYREAL *) mycalloc(data->loci, sizeof(MYREAL));
   freq = (MYREAL ***) mycalloc(data->numpop, sizeof(MYREAL **));
   doublevec2d(&total,data->numpop,data->loci);
   for (pop = 0; pop < data->numpop; pop++)
@@ -1924,57 +2150,96 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
   // calculate spectrum
   for (pop1 = 0; pop1 < data->numpop; pop1++)
     {
-      pop = options->newpops[pop1]-1;
+      //pop = options->newpops[pop1]-1;
       for (ind = 0; ind < data->numind[pop1][0]; ind++)
         {
 	  for (locus = 0; locus < data->loci; locus++)
 	    {
 	      thisallele = data->yy[pop1][ind][locus][0];
 	      found = findAllele(data, thisallele, locus);
-	      thatallele = data->yy[pop1][ind][locus][1];
-	      if(!strcmp(thisallele,thatallele))
+	      if(second)
 		{
-		     if(!strchr(thisallele,'?'))
-		       {
-			 freq[pop][locus][found] += 2.0 ;
-			 total[pop][locus] += 2.0 ;
-		       }
+		  thatallele = data->yy[pop1][ind][locus][1];
+		  if(!strcmp(thisallele,thatallele))
+		    {
+		      if(!strchr(thisallele,'?'))
+			{
+			  freq[pop1][locus][found] += two ;
+			  //total[pop][locus] += two ;
+			}
+		    }
+		  else	    
+		    {
+		      if(!strchr(thisallele,'?'))
+			{
+			  freq[pop1][locus][found] += one ;
+			  //total[pop][locus] += one ;
+			}
+		      found = findAllele(data, thatallele, locus);
+		      if(!strchr(thatallele,'?'))
+			{
+			  freq[pop1][locus][found] += one ;
+			  //total[pop][locus] += one ;
+			}
+		    }
 		}
 	      else
 		{
 		  if(!strchr(thisallele,'?'))
 		    {
-		      freq[pop][locus][found] += 1.0 ;
-		      total[pop][locus] += 1.0 ;
-		    }
-		  found = findAllele(data, thatallele, locus);
-		  if(!strchr(thatallele,'?'))
-		    {
-		      freq[pop][locus][found] += 1.0 ;
-		      total[pop][locus] += 1.0 ;
+		      freq[pop1][locus][found] += one ;
+		      //total[pop][locus] += one ;
 		    }
 		}
 	    }
 	}
     }
-  for (pop = 0; pop < options->newpops_numpop; pop++)
+  // now we calculate the poptotals and popfreqs if present
+  // this will change freq and total therefore transform the
+  // location score to population scores that are repeated
+  for (pop1 = 0; pop1 < data->numpop; pop1++)
     {
+      pop = options->newpops[pop1]-1;
+      if (pop == pop1)
+	continue;
       for (locus = 0; locus < data->loci; locus++)
 	{
 	  for(a=0; a < maxalleles[locus]; a++)
 	    {
-	      freq[pop][locus][a] /= (MYREAL) total[pop][locus];
+	      freq[pop][locus][a] += freq[pop1][locus][a];
+	    }
+	  for(a=0; a < maxalleles[locus]; a++)
+	    {
+	      freq[pop1][locus][a] = freq[pop][locus][a];
 	    }
 	}
     }
+  // now we have collected all numbers of alleles for each locus and each location
+  // we calculate now all the totals for each locality, 
+  for (pop = 0; pop < data->numpop; pop++)
+    {
+      for (locus = 0; locus < data->loci; locus++)
+	{
+	  loctotal = 0;
+	  for(a=0; a < maxalleles[locus]; a++)
+	    {
+	      loctotal += freq[pop][locus][a];
+	    }
+	  total[pop][locus] = loctotal;
+	  grandtotal[locus] += loctotal;
+	}
+    }
+
   // print in ascii
   //fprintf(stdout,"Allele frequency spectra\n");
   fprintf(outfile,"Allele frequency spectra\n");
   fprintf(outfile,"========================\n\n");
+  avghet1=0.0;
   for (locus = 0; locus < data->loci; locus++)
     {
       fprintf(outfile,"Locus %li\n", locus + 1);
       fprintf(outfile,"Allele  ");
+      general_homo = 0.0;
       for (pop1 = 0; pop1 < data->numpop; pop1++)
 	{
 	  maxallelepop[pop1]=0;
@@ -1994,32 +2259,76 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
 	      pop = options->newpops[pop1]-1;
 	      if(freq[pop][locus][a]>0.0)
 		{
-		  maxallelepop[pop] += 1;
-		  fprintf(outfile," %1.3f ",freq[pop][locus][a]);
+		  maxallelepop[pop1] += 1;
+		  //		  maxallelepop[pop] += 1.0/data->numpop;
+		  fprintf(outfile," %1.3f ",freq[pop][locus][a]/total[pop][locus]);
 		  allfreq += freq[pop][locus][a];
+		  //		  allfreq += freq[pop][locus][a];
 		}
 	      else
 		{
 		  fprintf(outfile,"   -   ");
 		}
 	    }
-	  fprintf(outfile," %1.3f\n", (MYREAL) allfreq/data->numpop);
+	  fprintf(outfile," %1.3f\n", (MYREAL) allfreq/grandtotal[locus]);
+	  fx  =  allfreq/grandtotal[locus];
+	  general_homo += fx * fx;
+	  //	  fprintf(outfile," %1.3f %f\n", fx*fx,general_homo);
 #ifdef BEAGLE
 	  // debug not ready yet for subloci
 	  world->mutationmodels[world->sublocistarts[locus]].basefreqs[a] = allfreq/data->numpop;
 #endif
 	}
-      fprintf(outfile,"Total ");
+      fprintf(outfile,"Total  ");
+      for (pop1 = 0; pop1 < data->numpop; pop1++)
+	{
+	  // pop = options->newpops[pop1]-1;
+	  fprintf(outfile," %5li ",  maxallelepop[pop1]);
+	}
+      fprintf(outfile," %5li\n", maxalleles[locus]);
+
+      if(maxalleles[locus]<=1 && strchr(SNPTYPES,options->datatype))
+	{
+	  data->skiploci[locus] = TRUE;
+	}
+
+      fprintf(outfile,"H_exp  ");
       for (pop1 = 0; pop1 < data->numpop; pop1++)
 	{
 	  pop = options->newpops[pop1]-1;
-	  fprintf(outfile," %5li ",maxallelepop[pop]);
+	  homo = 0.0;	  
+	  for(a=0;a<maxalleles[locus];a++)
+	    {
+	      f = freq[pop][locus][a]/total[pop][locus];
+	      homo += f*f;
+	    }
+	  v = 1.0 - homo;
+	  avghet[pop1] += v;
+	  fprintf(outfile," %5.3f ",v);
 	}
-      fprintf(outfile," %5li\n\n", maxalleles[locus]);
+      fprintf(outfile," %5.3f\n\n",1.0-general_homo);
+      avghet1 += 1.0 - general_homo;
+      //      fprintf(outfile," %5.3f\n\n",avghet1/data->numpop);
     }
+  fprintf(outfile,"Average expected heterozygosity\n");
+  for (pop1 = 0; pop1 < data->numpop; pop1++)
+    {
+      pop = options->newpops[pop1]-1;
+      fprintf(outfile,"Pop%-2li  ",pop+1);
+    }
+  fprintf(outfile,"All\n");
+  for (pop = 0; pop < data->numpop+1; pop++)
+    fprintf(outfile,"-------");
+  fprintf(outfile, "\n");
+  for (pop1 = 0; pop1 < data->numpop; pop1++)
+    {
+      pop = options->newpops[pop1]-1;
+      fprintf(outfile,"%5.3f  ",avghet[pop1] / data->loci);
+    }
+  fprintf(outfile,"%5.3f\n\n",avghet1/data->loci);
   // printd in PDF
 #ifdef PRETTY
-  pdf_print_spectra(data, freq, maxalleles);
+  pdf_print_spectra(data, options, freq, total, grandtotal, avghet, avghet1, maxalleles);
 #endif
   fflush(outfile);
   // cleanup
@@ -2035,7 +2344,9 @@ void print_spectra(world_fmt * world, option_fmt * options,data_fmt * data)
   myfree(freq);
   myfree(maxalleles);
   myfree(maxallelepop);
+  myfree(avghet);
   free_doublevec2d(total);
+  myfree(grandtotal);
 }
 
 void
@@ -2271,7 +2582,7 @@ void find_allele_repeatlength(data_fmt *data, option_fmt *options, long locus)
 	      nonfloored = (value - keepstartvalue)/data->repeatlength[locus]; 
 	      floored = floor(nonfloored);
 	      diff = nonfloored - floored; 
-	      intval = (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
+	      intval = MSAT_OFFSET + (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
 	      sprintf(data->yy[pop][ind][locus][0],"%li",intval);
 	    }
 	  if (a2[0]!='?')
@@ -2280,7 +2591,7 @@ void find_allele_repeatlength(data_fmt *data, option_fmt *options, long locus)
 	      nonfloored = (value - keepstartvalue)/data->repeatlength[locus]; 
 	      floored = floor(nonfloored);
 	      diff = nonfloored - floored; 
-	      intval = (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
+	      intval =  MSAT_OFFSET + (long) floored + (RANDUM() < (1.0-diff) ? 0 : 1); 
 	      sprintf(data->yy[pop][ind][locus][1],"%li",intval);
 	    }
 	}
@@ -2294,11 +2605,12 @@ create_alleles (data_fmt * data, option_fmt *options)
   long n=0;
   MYREAL  mean=0.;
   MYREAL  delta=0.;
-    long locus, pop, ind;
-    long z;
-    char a1[DEFAULT_ALLELENMLENGTH];
-    char a2[DEFAULT_ALLELENMLENGTH];
-    for (locus = 0; locus < data->loci; locus++)
+  long locus, pop, ind;
+  long z;
+  char a1[LINESIZE];//DEFAULT_ALLELENMLENGTH];
+  char a2[LINESIZE];//DEFAULT_ALLELENMLENGTH];
+  boolean second = (strchr(ALLELETYPES,options->datatype)!=NULL);
+  for (locus = 0; locus < data->loci; locus++)
     {
       if(data->repeatlength[locus]!=0)
 	find_allele_repeatlength(data, options,locus);
@@ -2307,18 +2619,26 @@ create_alleles (data_fmt * data, option_fmt *options)
         {
             for (ind = 0; ind < data->numind[pop][locus]; ind++)
             {
+
                 strcpy (a1, data->yy[pop][ind][locus][0]);
-                strcpy (a2, data->yy[pop][ind][locus][1]);
-                if (!strcmp (a1, a2))
-                {
-                    addAllele (data, a1, locus, &z);
-                }
-                else
-                {
-                    addAllele (data, a1, locus, &z);
-                    addAllele (data, a2, locus, &z);
-                }
-            }
+		if(second)
+		  {
+		    strcpy (a2, data->yy[pop][ind][locus][1]);
+		    if (!strcmp (a1, a2))
+		      {
+			addAllele (data, a1, locus, &z);
+		      }
+		    else
+		      {
+			addAllele (data, a1, locus, &z);
+			addAllele (data, a2, locus, &z);
+		      }
+		  }
+		else
+		  {
+		    addAllele (data, a1, locus, &z);
+		  }
+	    }
         }
 	if(z==0)
 	  {
