@@ -19,12 +19,12 @@ Copyright 2002 Peter Beerli and Joseph Felsenstein
 Copyright 2003-2008 Peter Beerli
 Copyright 2009-2010 Peter Beerli and Michal Palczewski
  
-$Id: definitions.h 1962 2011-10-09 21:11:45Z beerli $
+$Id: definitions.h 2033 2012-05-11 15:21:41Z beerli $
  
   *----------------------------------------------------------------
   */
 #ifndef MIGRATEVERSION
-#define MIGRATEVERSION "3.2.17"
+#define MIGRATEVERSION "3.2.19"
 #endif
 #ifndef MIGRATESUBVERSION
 #define MIGRATESUBVERSION "x"
@@ -396,6 +396,8 @@ long chains is used */
 #define ATOI(a) strtol((a),(char **) NULL, 10)
 #define ATOF(a) strtod((a),(char **) NULL, 10)
 
+/* mutation model setting */
+#define MSAT_OFFSET 10
 /* prior setting */
 #define SLICE        5
 #define GAMMAPRIOR   4
@@ -505,6 +507,7 @@ long chains is used */
 #define PAGEFEED               fprintf(outfile,"\n\f\n")
 #define PAGEFEEDWORLD               fprintf(world->outfile,"\n\f\n")
 #define PAGEFEEDW                   fprintf(universe[0]->outfile,"\n\f\n")
+#define LINELENGTH           90    /* number of print positions*/
 #define START                  0
 #define STOP                   1
 #define CONT                   3
