@@ -11,12 +11,13 @@
 #include "sighandler.h"
 #include "reporter.h"
 #include "pretty.h"
-
+#include <errno.h>
 extern int myID;
+extern int numcpu;
 #ifdef ZNZ
-void read_bayes_fromfile(znzFile mdimfile, world_fmt *world,option_fmt *options);
+void read_bayes_fromfile(znzFile fmdimfile, world_fmt *world,option_fmt *options, char **files, long fnum);
 #else
-void read_bayes_fromfile(FILE *mdimfile, world_fmt *world,option_fmt *options);
+void read_bayes_fromfile(FILE *fmdimfile, world_fmt *world,option_fmt *options, char **files, long fnum);
 #endif
 
 #ifdef ZNZ
@@ -136,197 +137,229 @@ long get_fullbinsum(MYREAL *lowerbound, MYREAL *upperbound, world_fmt *world, op
 /// read bayesallfile from disk and creates all the needed parts to 
 /// recreate the output and pdf output
 #ifdef ZNZ
-void read_bayes_fromfile(znzFile mdimfile, world_fmt *world,option_fmt *options)
+void read_bayes_fromfile(znzFile fmdimfile, world_fmt *world,option_fmt *options, char **files, long fnum)
 #else
-void read_bayes_fromfile(FILE *mdimfile, world_fmt *world,option_fmt *options)
+  void read_bayes_fromfile(FILE *fmdimfile, world_fmt *world,option_fmt *options, char **files, long fnum)
 #endif
 {
-
+  const long nn = world->numpop2 + world->bayes->mu * world->loci + 1;// One is for Log(Prob(Data|Model)
   long *n = NULL;
-  const long nn = world->numpop2 + world->bayes->mu * world->loci + 1;// One is for Log(Prob(Data|Model)                                    
-  long j0;
-  long j;
-  //  char *input;
+  long j0, j;
   long step;
   long locus;
-  long numpop = world->numpop;
-  long numpop2 = world->numpop2;
   long frompop;
   long topop;
- // long replicate;
- // long T ; // was world->treetimes->T, but this construct is freed, we do not need to
-  // keep this values as it only used for the output into bayesallfile
- // MYREAL treelength;
-  MYREAL post;
-  MYREAL like;
-//  MYREAL probg;
-//  MYREAL prior;
-  MYREAL *params;
-#ifdef BFDEBUG
   long nnn=0;
   long t;
-//  MYREAL lsum;
-  long hc = world->options->heated_chains; 
-#endif
+  long bin;
+  const long numpop = world->numpop;
+  const long numpop2 = world->numpop2;
+  const long np = world->numpop2 + world->bayes->mu;
+  const long hc = world->options->heated_chains; 
+  long numbins = 0;
+  long numbinsall = 0;
   char *input;
   char *inptr;
   bayes_fmt * bayes = world->bayes;
   bayeshistogram_fmt *hist;
-  long bin;
+  MYREAL post;
+  MYREAL like;
+  MYREAL *params;
   MYREAL *delta = bayes->deltahist;
-  //MYREAL *delta; 
   MYREAL *lowerbound;
   MYREAL *upperbound;
   MYREAL *autocorrelation;
   MYREAL *ess;
-  //  long binwidth;
-  long numbins = 0;
-  long numbinsall = 0;
-  //MYREAL m;
   boolean *done;
   boolean recordedusem = options->usem;
-  const long np = world->numpop2 + world->bayes->mu;
-
+#ifndef ZNZ
+  FILE *mdimfile = fmdimfile;
+#else
+  znzFile mdimfile = fmdimfile;
+#endif
+  long f;
   done = (boolean *) mycalloc(world->loci, sizeof(boolean));
-  // params can be replaced by a single value for param and rate
   params = (MYREAL *) mycalloc(2+ numpop2 + world->bayes->mu, sizeof(MYREAL));
   autocorrelation = (MYREAL *) mycalloc(2 * world->loci * nn, sizeof(MYREAL));
   ess = autocorrelation + world->loci * nn;
   lowerbound = (MYREAL *) mycalloc(nn, sizeof(MYREAL));
   upperbound = (MYREAL *) mycalloc(nn, sizeof(MYREAL));
-  //delta = (MYREAL *) mycalloc(nn-1, sizeof(MYREAL));
   n = (long *) mycalloc(numpop2 + world->bayes->mu * world->loci, sizeof(long));
   input = (char *) mycalloc(SUPERLINESIZE, sizeof(char));
   printf("Begin reading the bayesallfile back into the system\n");
-#ifdef ZNZ
-  unsigned long bytes = SUPERLINESIZE > ONEMEGABYTE ? SUPERLINESIZE : ONEMEGABYTE;
-  znzbuffer(mdimfile,bytes);
-  while(ZNZFGETS(input,SUPERLINESIZE,mdimfile) != EOF)
-#else
-  while(FGETS(input,SUPERLINESIZE,mdimfile) != EOF)
-#endif
+  // files has always at least one entry
+  if (files!=NULL)
     {
-      printf("%s\n",input);
-      // grab the commentlines
-      while(input[0]=='#' || input[0]=='S')
-	{
+      mdimfile = NULL;
+    }
+  else
+    {
+      error("Filelist should be filled\n");
+    }
+#ifdef ZNZ 
+  unsigned long bytes = SUPERLINESIZE > ONEMEGABYTE ? SUPERLINESIZE : ONEMEGABYTE;
+#endif
+  for(f=0;f<fnum;f++)
+    {
+      input[0]='\0';
 #ifdef ZNZ
-	  ZNZFGETS(input,SUPERLINESIZE,mdimfile);
+	  mdimfile = znzopen(files[f], "r", options->use_compressed);      
 #else
-	  FGETS(input,SUPERLINESIZE,mdimfile);
-	  printf("#### %s\n",input);
+	  mdimfile = fopen(files[f],"r");      
 #endif
-	}
-      // read the bayesallfile
-       printf("%s\n",input);
-      if(input !=NULL)
-	{
-	  inptr = input;
-	  step       = atol(strsep(&inptr,"\t"));
-	  printf("%li\n",step);
-	  locus      = atol(strsep(&inptr,"\t"))-1;
-	  if(locus == -1)
-	    error("help");
-	//  replicate  = atol(strsep(&inptr,"\t"))-1;
-      (void) strsep(&inptr,"\t");
-	  post       = atof(strsep(&inptr,"\t"));
-	  like       = atof(strsep(&inptr,"\t"));
-	//  probg      = atof( strsep(&inptr,"\t"));
-       (void) strsep(&inptr,"\t");
-	//  prior      = atof( strsep(&inptr,"\t"));
-       (void) strsep(&inptr,"\t");
-	//  T = atol(strsep(&inptr,"\t"))+1;
-       (void) strsep(&inptr,"\t");
-	//  treelength = atof(strsep(&inptr,"\t"));
-       (void) strsep(&inptr,"\t");
-	  params[0] = post;
-	  params[1] = like;
-
-	  if(!done[locus])
+	  printf("%s opened\n",files[f]);
+	  if (mdimfile == NULL)
 	    {
-
-	      done[locus] = TRUE;
-	      // allocate the number of bins for the histogram
-	      bayes->histogram[locus].binsum = get_fullbinsum(lowerbound, upperbound, world, options, locus);
-	      bayes->histogram[locus].results = (MYREAL *) mycalloc(bayes->histogram[locus].binsum + 1, sizeof(MYREAL));
-	      bayes->histogram[locus].set95 = (char *) mycalloc(bayes->histogram[locus].binsum* 2 + 2, sizeof(char));
-	      bayes->histogram[locus].set50 = world->bayes->histogram[locus].set95 + bayes->histogram[locus].binsum + 1;
-	      memset(world->bayes->histogram[locus].results, 0 , sizeof(MYREAL) * (world->bayes->histogram[locus].binsum)); 
-   
+	      printf("errno = %s (%d).\n", strerror(errno), errno);
+	      exit(1);
 	    }
-	  hist = &bayes->histogram[locus];
-	  numbinsall = 0;
-	  for(j0=0;j0 < numpop2; j0++)
-	    {
-	      if(bayes->map[j0][1] == INVALID)
-		continue;
-	      else
+#ifdef ZNZ
+      znzbuffer(mdimfile,bytes);
+      while(ZNZFGETS(input,SUPERLINESIZE,mdimfile) != EOF)
+#else
+      while(FGETS(input,SUPERLINESIZE,mdimfile) != EOF)
+#endif
+	  {
+	    //printf("%s> ",files[f]);//,input);
+	  // grab the commentlines
+	    if (input[0] == '#' || input[0]=='S' || input[0]=='\0')
+	      continue;
+	    //	    while(input[0]=='#' || input[0]=='S')
+	    //{
+	    //#ifdef ZNZ
+	      // ZNZFGETS(input,SUPERLINESIZE,mdimfile);
+	    //#else
+	      //FGETS(input,SUPERLINESIZE,mdimfile);
+	      //printf("#### %s\n",input);
+	    //#endif
+	    //}
+	  // read the bayesallfile
+	  //	  printf("%s\n",input);
+	  //if(input[0] == '\0')
+	  //  continue;
+	    //	  if(input !=NULL && input[0] != '\0')
+	    //{
+	      inptr = input;
+	      
+	      step       = atol(strsep(&inptr,"\t"));
+	      //printf("%li\n",step);
+	      locus      = atol(strsep(&inptr,"\t"))-1;
+	      if(locus == -1)
+		error("help");
+	      //  replicate  = atol(strsep(&inptr,"\t"))-1;
+	      (void) strsep(&inptr,"\t");
+	      post       = atof(strsep(&inptr,"\t"));
+	      like       = atof(strsep(&inptr,"\t"));
+	      //  probg      = atof( strsep(&inptr,"\t"));
+	      (void) strsep(&inptr,"\t");
+	      //  prior      = atof( strsep(&inptr,"\t"));
+	      (void) strsep(&inptr,"\t");
+	      //  T = atol(strsep(&inptr,"\t"))+1;
+	      (void) strsep(&inptr,"\t");
+	      //  treelength = atof(strsep(&inptr,"\t"));
+	      (void) strsep(&inptr,"\t");
+	      params[0] = post;
+	      params[1] = like;
+	      
+	      if(!done[locus])
 		{
-		  j = bayes->map[j0][1];
+		  
+		  done[locus] = TRUE;
+		  // allocate the number of bins for the histogram
+		  bayes->histogram[locus].binsum = get_fullbinsum(lowerbound, upperbound, world, options, locus);
+		  bayes->histogram[locus].results = (MYREAL *) mycalloc(bayes->histogram[locus].binsum + 1, sizeof(MYREAL));
+		  bayes->histogram[locus].set95 = (char *) mycalloc(bayes->histogram[locus].binsum* 2 + 2, sizeof(char));
+		  bayes->histogram[locus].set50 = world->bayes->histogram[locus].set95 + bayes->histogram[locus].binsum + 1;
+		  memset(world->bayes->histogram[locus].results, 0 , sizeof(MYREAL) * (world->bayes->histogram[locus].binsum)); 
+		  
 		}
-	      if(j < j0)
+	      hist = &bayes->histogram[locus];
+	      numbinsall = 0;
+	      for(j0=0;j0 < numpop2; j0++)
 		{
-		  continue;
-		}
-	      else
-		{
-		  params[j+2] =  atof(strsep(&inptr,"\t"));
-		  if(j>=numpop && options->usem!=recordedusem)
+		  if(bayes->map[j0][1] == INVALID)
+		    continue;
+		  else
 		    {
-		      m2mm(j, numpop,&frompop,&topop);
-		      if(recordedusem)// recorded bayesallfile was with M, new request is for 4Nm 
-			{
-			  params[j+2] *= params[topop+2];
-			}
-		      else //recorded bayesallfile was with xNm, new request is for M
-			{
-			  params[j+2] /= params[topop+2];
-			}
+		      j = bayes->map[j0][1];
 		    }
-		  n[j] += 1;
-		  hist->means[j] += (params[j+2] - hist->means[j]) / n[j];
+		  if(j < j0)
+		    {
+		      continue;
+		    }
+		  else
+		    {
+		      params[j+2] =  atof(strsep(&inptr,"\t"));
+		      if(j>=numpop && options->usem!=recordedusem)
+			{
+			  m2mm(j, numpop,&frompop,&topop);
+			  if(recordedusem)// recorded bayesallfile was with M, new request is for 4Nm 
+			    {
+			      params[j+2] *= params[topop+2];
+			    }
+			  else //recorded bayesallfile was with xNm, new request is for M
+			    {
+			      params[j+2] /= params[topop+2];
+			    }
+			}
+		      n[j] += 1;
+		      hist->means[j] += (params[j+2] - hist->means[j]) / n[j];
+		    }
+		  numbinsall += hist->bins[j];
+		  numbins = numbinsall - hist->bins[j];
+		  bin = (long) ((params[j+2]-lowerbound[j]) / delta[j]);
+		  hist->minima[j0] = lowerbound[j];
+		  hist->maxima[j0] = upperbound[j];
+		  hist->results[numbins + bin] += 1.;
+		  bayes->histtotal[locus * np + j] += 1;
 		}
-	      numbinsall += hist->bins[j];
-	      numbins = numbinsall - hist->bins[j];
-	      bin = (long) ((params[j+2]-lowerbound[j]) / delta[j]);
-	      hist->minima[j0] = lowerbound[j];
-	      hist->maxima[j0] = upperbound[j];
-	      hist->results[numbins + bin] += 1.;
-	      bayes->histtotal[locus * np + j] += 1;
-	    }
-	  if(bayes->mu)
-	    {
-	      numbins = numbinsall;
-	      params[j0+2] = atof(strsep(&inptr,"\t"));
-	      n[j0+locus] += 1;
-	      hist->means[j0] += (params[j0+2] - hist->means[j0]) / n[j0+locus];
-	      bin = (long) ((params[j0+2]-lowerbound[j0]) / delta[j0]); 
-	      hist->minima[j0] = lowerbound[j0];
-	      hist->maxima[j0] = upperbound[j0];
-	      hist->results[numbins + bin] += 1.;
-	      bayes->histtotal[locus * np + j0] += 1;
-	    }
-#ifdef BFDEBUG
-	  if(world->options->datatype == 'g')
-	    {
-	      for(t=0;t<hc;t++)
+	      if(bayes->mu)
 		{
-		  world->bf[locus * hc + t] = atof(strsep(&inptr,"\t"));
+		  numbins = numbinsall;
+		  params[j0+2] = atof(strsep(&inptr,"\t"));
+		  n[j0+locus] += 1;
+		  hist->means[j0] += (params[j0+2] - hist->means[j0]) / n[j0+locus];
+		  bin = (long) ((params[j0+2]-lowerbound[j0]) / delta[j0]); 
+		  hist->minima[j0] = lowerbound[j0];
+		  hist->maxima[j0] = upperbound[j0];
+		  hist->results[numbins + bin] += 1.;
+		  bayes->histtotal[locus * np + j0] += 1;
+		}
+	      if(world->options->datatype == 'g')
+		{
+		  for(t=0;t<hc;t++)
+		    {
+		      world->bf[locus * hc + t] = atof(strsep(&inptr,"\t"));
+		    }
+		  // dummy read of thermo sum up to this point
+		  //lsum = atof(strsep(&inptr,"\t"));
+		  (void ) strsep(&inptr,"\t");
+		  // harmonic mean: scaler contains the log value, hm contains 1.
+		  world->hmscale[locus] = atof(strsep(&inptr,"\t"));
+		  world->hm[locus] = 1.; 
+		  //	      calculate_ess_frombayes (world, step, params, locus, &autocorrelation[locus*nn], &ess[locus*nn]);
+		  calculate_ess_frombayes (world, step, params, locus, autocorrelation, ess);
+		}
+	      else
+		{
+		  //		  fprintf(stdout,"%s\n",inptr);
+		  (void) strsep(&inptr,"\n");
 		}
-	      // dummy read of thermo sum up to this point
-	      //lsum = atof(strsep(&inptr,"\t"));
-          (void ) strsep(&inptr,"\t");
-	      // harmonic mean: scaler contains the log value, hm contains 1.
-	      world->hmscale[locus] = atof(strsep(&inptr,"\t"));
-	      world->hm[locus] = 1.; 
-	      //	      calculate_ess_frombayes (world, step, params, locus, &autocorrelation[locus*nn], &ess[locus*nn]);
-	      calculate_ess_frombayes (world, step, params, locus, autocorrelation, ess);
 	    }
+      //}
+#ifdef ZNZ
+      znzclose(mdimfile);
+#else
+      fclose(mdimfile);
 #endif
+      if(options->mdimdelete)
+	{
+	  remove(options->bayesmdimfilename);
 	}
+      fprintf(stdout,"%s closed\n",files[f]);
+      mdimfile=NULL;
     }
   printf("End reading the bayesallfile back into the system\n");
-#ifdef BFDEBUG
   if(world->options->datatype == 'g')
     {
       // reset the archiving machinery
@@ -341,12 +374,11 @@ void read_bayes_fromfile(FILE *mdimfile, world_fmt *world,option_fmt *options)
 	      world->auto_archive[t] += (autocorrelation[j*nn + t] - world->auto_archive[t])/nnn;
 	      // summing ess values
 	      world->ess_archive[t] += ess[j*nn + t];
-	      printf("j=%li t=%li %f\n", j, t, world->ess_archive[t]);
+	      //printf("j=%li t=%li %f\n", j, t, world->ess_archive[t]);
 	    }
 	  nnn++;
 	}
     }
-#endif /*BFDEBUG*/
 
   myfree(params);
   myfree(autocorrelation);
