@@ -18,7 +18,7 @@
  and is copyrighted. Of course, we do not guarantee that the software
  works and are not responsible for any damage you may cause or have.
  
- $Id: mcmc1.c 1926 2011-06-11 16:58:42Z beerli $
+ $Id: mcmc1.c 2035 2012-05-16 20:03:33Z beerli $
  -------------------------------------------------------*/
 /* \file mcmc1.c
  
@@ -75,7 +75,7 @@ void chooseTarget (proposal_fmt * proposal, timelist_fmt * timevector,
 void findbordernodes (node * theNode, proposal_fmt * proposal, long pop,
                       node *** bordernodes, long *allocsize, long *bordernum, vtlist ** tyme,
                       long gte);
-void free_proposal (proposal_fmt * proposal);
+void free_masterproposal (proposal_fmt * proposal);
 void free_timevector (timelist_fmt * timevector);
 void prune_timelist (timelist_fmt * oldtv, timelist_fmt * newtv, register node ** __restrict ptr, proposal_fmt * proposal, long numpop);
 
@@ -239,7 +239,9 @@ tree_update (world_fmt * world, long g)
         {
             fprintf(stdout,"%i> Proposal failed because of unordered entry in time list, abort this sample\nProposed time=%f origin time=%f nexttime=%f\n", myID, proposal->time, proposal->origin->tyme, nexttime);
             // we end up here when the migration events exceed the upper limit
-            free_proposal (proposal);
+#ifndef TESTING2
+            free_masterproposal (proposal);
+#endif
             free_timevector (timevector);
             return 0;
             //	    error("Proposal of time failed\n");
@@ -251,7 +253,9 @@ tree_update (world_fmt * world, long g)
                 if (!migrate (proposal, proposal->origin))
                 {
                     // we end up here when the migration events exceed the upper limit
-                    free_proposal (proposal);
+#ifndef TESTING2
+                    free_masterproposal (proposal);
+#endif
                     free_timevector (timevector);
                     //warning("Event was migration event but could not match a lineage to it\n");
                     return 0;
@@ -268,7 +272,9 @@ tree_update (world_fmt * world, long g)
                     if (!migrate (proposal, proposal->origin))
                     {
                         // we end up here when the migration events exceed the upper limit
-                        free_proposal (proposal);
+#ifndef TESTING2
+                        free_masterproposal (proposal);
+#endif
                         free_timevector (timevector);
                         return 0;
                     }
@@ -307,7 +313,9 @@ tree_update (world_fmt * world, long g)
     {
         if (!pre_population(proposal, (*timevector).tl, (*timevector).T - 1, &slider))
         {
-            free_proposal (proposal);
+#ifndef TESTING2
+            free_masterproposal (proposal);
+#endif
             free_timevector (timevector);
             return 0;
         }
@@ -386,7 +394,9 @@ tree_update (world_fmt * world, long g)
         world->migration_counts = 0;
         /* report the number of migration on the tree */
         count_migrations (world->root->next->back, &world->migration_counts);
-        free_proposal (proposal);
+#ifndef TESTING2
+        free_masterproposal (proposal);
+#endif
         free_timevector (timevector);
         return 1;   /* new tree accepted */
     }
@@ -400,7 +410,9 @@ tree_update (world_fmt * world, long g)
         //traverse_check(crawlback (proposal->root->next));printf(".");
         //
     }
-    free_proposal (proposal);
+#ifndef TESTING2
+    free_masterproposal (proposal);
+#endif
     free_timevector (timevector);
     return 0;   /* not accepted */   
 }
@@ -1504,10 +1516,10 @@ pre_population (proposal_fmt * proposal, vtlist * ltime, long gte,
 ///
 /// freeing the proposal structure, this is the replacement of the real free_proposal method
 /// and does not allocate, but reuses old memory
-void free_proposal(proposal_fmt *proposal)
-{
+//void free_proposal(proposal_fmt *proposal)/
+//{
     // do nothing
-}
+//}
 void reset_simple_proposal_variables(proposal_fmt **proposal)
 {
     (*proposal)->mig_removed = FALSE;
@@ -1602,12 +1614,12 @@ reset_proposal (proposal_fmt ** proposal, world_fmt *world)
     }
 #endif
 }
-#else
+#endif
 ///
 /// freeing the proposal structure, this will be replaced by a function that resets permanently
 /// allocated structure [reset_proposal()]
 void
-free_proposal (proposal_fmt * proposal)
+free_masterproposal (proposal_fmt * proposal)
 {
     //static long count=0;
     myfree(proposal->aboveorigin);
@@ -1646,7 +1658,7 @@ free_proposal (proposal_fmt * proposal)
 #endif
     myfree(proposal);
 }
-#endif /*TESTING*/
+//#endif /*TESTING*/
 
 
 void
@@ -1819,13 +1831,12 @@ eventtime (proposal_fmt * proposal, long pop, vtlist * tentry, char *event)
     MYREAL  denom;
     MYREAL  invdenom;
     MYREAL  rate = proposal->world->options->mu_rates[proposal->world->locus];
-    MYREAL  inheritance = proposal->world->options->inheritance_scalars[proposal->world->locus];
     MYREAL  invrate = 1./ rate;
     MYREAL  mm         = proposal->mig0list[pop] * invrate ;
     //long    design0    = proposal->design0list[pop];
     //could be used to calculate p(gn|go): MYREAL nu;
     lines    = 2.0 * tentry->lineages[pop];
-    denom    = mm + (lines * (1.0 / (inheritance*proposal->param0[pop]*rate)));
+    denom    = mm + (lines * (1.0 / (proposal->param0[pop]*rate)));
     invdenom = 1.0 / denom;
     interval =  (-(LOG (/*nu = */UNIF_RANDUM ())) * invdenom) ;
     //could be used to calculate p(gn|go): proposal->nu += nu; //adds the probabilities (=random numbers);
