@@ -16,7 +16,7 @@
  
   Copyright 2002 Peter Beerli and Joseph Felsenstein, Seattle WA
   Copyright 2003 Peter Beerli, Tallahassee FL
-  $Id: migration.h 1941 2011-08-07 23:08:22Z beerli $
+  $Id: migration.h 2017 2012-03-19 22:38:48Z beerli $
   *----------------------------------------------------------------
 */
 /**
@@ -469,6 +469,7 @@ typedef struct _option
   char *datefilename;
   char *bayesfilename;
   char *bayesmdimfilename;
+  boolean mdimdelete;
   int use_compressed;
   boolean prioralone;
 
@@ -553,6 +554,7 @@ typedef struct _option
   long *newpops;
   long newpops_numalloc;
   long newpops_numpop;
+  //  boolean poprelabeled;
   MYREAL *inheritance_scalars;
   long inheritance_scalars_numalloc;
   boolean has_datefile;
