@@ -22,7 +22,7 @@ Copyright 2003-2007 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: random.c 1892 2011-05-26 17:00:58Z beerli $
+$Id: random.c 1983 2011-11-23 14:14:28Z beerli $
 -------------------------------------------------------*/
 /* \file random.c */
 #include "sighandler.h"
@@ -46,7 +46,7 @@ $Id: random.c 1892 2011-05-26 17:00:58Z beerli $
 #endif
 
 
-#ifdef SNOWLEOPARD
+#ifdef GRANDCENTRAL
 #include <dispatch/dispatch.h>
 #endif
 
@@ -643,7 +643,7 @@ randum (void)
     fprintf(stdout,"%i> R:%f\n",myID,val);
     return val;
 #else
-#ifdef SNOWLEOPARD
+#ifdef GRANDCENTRAL
     //    MYREAL r;
     // dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
     //dispatch_async(queue,
