@@ -218,10 +218,10 @@ void calculate_expected_values(tetra **eventbins,
 {
   //  static MYREAL maxlweight = -HUGE;
   //  MYREAL oldlweight = -HUGE;
-  boolean change;
+//xcode  boolean change;
   //  long p;
 
-  float lweight;
+//xcode  float lweight;
   //  float unweight;
   long i,j;
   long pop;
@@ -244,8 +244,8 @@ void calculate_expected_values(tetra **eventbins,
   float binage  = eventinterval * bin; //this is on the left side of the bin
   MYREAL firstbinage        = eventinterval * firstbin; //this is the age at the left side of first bin
   tetra *bins=NULL;
-  change =FALSE;
-  lweight = 1.0;
+  //xcode change =FALSE;
+  //xcode lweight = 1.0;
   // do we need to report problems when bin number is above 10000?
   // 
   //  if(age > 1000.0 || bin > 10000)
@@ -440,12 +440,12 @@ void read_expected_values_fromfile(FILE *file, world_fmt *world)
   long locus;
   long pop;
   long i;
-  MYREAL age;
+ // MYREAL age;
   char *input;
   char *inptr;
   tetra **eventbins;
   long *eventbinnum;
-  MYREAL eventbinsize;
+//xcode  MYREAL eventbinsize;
   long spacer;
   long * allocsize;
   allocsize = (long *) mycalloc(world->numpop2*world->loci,sizeof(long));
@@ -471,11 +471,12 @@ void read_expected_values_fromfile(FILE *file, world_fmt *world)
 	    error("help");
 	  pop       = atol(strsep(&inptr,"\t"))-1;
 	  i          = atol(strsep(&inptr,"\t"))-1;
-	  age        = atof(strsep(&inptr,"\t"));
+	  //xcode age        = atof(strsep(&inptr,"\t"));
+      (void) strsep(&inptr,"\t");
 
 	  eventbins =  world->mighistloci[locus].eventbins;
 	  eventbinnum = world->mighistloci[locus].eventbinnum;
-	  eventbinsize = world->mighistloci[locus].eventbinsize;
+	  //xcode eventbinsize = world->mighistloci[locus].eventbinsize;
 	  eventbinnum[pop] = i+1;
 	  if(allocsize[spacer + pop] < eventbinnum[pop])
 	    {
@@ -483,10 +484,10 @@ void read_expected_values_fromfile(FILE *file, world_fmt *world)
 	      eventbins[pop] = (tetra *) myrealloc(eventbins[pop], allocsize[spacer + pop] * sizeof(tetra));
 	    }
 
-	  if(i==0)
-	    {
-	      eventbinsize = 2 * age;
-	    }
+	  //xcode if(i==0)
+	  //xcode   {
+	  //xcode     eventbinsize = 2 * age;
+	  //xcode   }
 	  eventbins[pop][i][0] = atof(strsep(&inptr,"\t"));
 	  eventbins[pop][i][1] = atof(strsep(&inptr,"\t"));
 	  eventbins[pop][i][2] = atof(strsep(&inptr,"\t"));
