@@ -20,7 +20,7 @@ Copyright 2003-2006 Peter Beerli, Tallahassee FL
  works and are not responsible for any damage you may cause or have.
  
  
-$Id: tools.c 1959 2011-09-15 19:40:04Z beerli $
+$Id: tools.c 2007 2012-02-19 18:16:49Z beerli $
 -------------------------------------------------------*/
 /* \file tools.c */
 
@@ -625,10 +625,22 @@ void
 init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 {
 	long *tempdb;
+	long maxfilenum = MYMAXFILENUM; //see definitions.h all files have a number and a filename;
 #ifdef MPI
 	long i,j;
+#ifdef PARALIO
+	long l;
+	boolean my_file_open_error;
+	char error_string[LINESIZE];
+	int length_of_error_string, error_class;
+	if(options->has_bayesmdimfile)
+	  {
+	    l = strlen(options->bayesmdimfilename);
+	    sprintf(options->bayesmdimfilename+l,".%d",myID);
+	  }
 #endif
-    long maxfilenum = MYMAXFILENUM; //see definitions.h all files have a number and a filename;
+#endif
+
     tempdb = (long *) mycalloc(maxfilenum,sizeof(long));
 	
     if (myID == MASTER)
@@ -763,15 +775,16 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 			    znzopenfile (&world->bayesmdimfile, options->bayesmdimfilename, "w", options->use_compressed);
 #else
 			    openfile (&world->bayesmdimfile, options->bayesmdimfilename, "w+",  NULL);
-#endif
-#endif			  
+#endif /*znz*/
+#endif /*not paralio*/			  
 #else
+			    // not MPI
 #ifdef ZNZ
 			    znzopenfile (&world->bayesmdimfile, options->bayesmdimfilename, "w",  options->use_compressed);
 #else
 			    openfile (&world->bayesmdimfile, options->bayesmdimfilename, "w+",  NULL);
 #endif
-#endif
+#endif /*mpi*/
 			  }
 			else
 			  {
@@ -788,11 +801,25 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 		    if(options->has_bayesmdimfile)
 		      {
 #ifdef PARALIO
-			MPI_File_open(comm_world,options->bayesmdimfilename,MPI_MODE_CREATE | MPI_MODE_WRONLY ,MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+			fprintf(stdout,"%i> before open %s\n",myID,options->bayesmdimfilename);
+			my_file_open_error = MPI_File_open(MPI_COMM_SELF, options->bayesmdimfilename, 
+							   MPI_MODE_CREATE | MPI_MODE_WRONLY ,
+							   MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+			if (my_file_open_error != MPI_SUCCESS) 
+			  {
+			    MPI_Error_class(my_file_open_error, &error_class);
+			    MPI_Error_string(error_class, error_string, &length_of_error_string);
+			    printf("%i> %s %s\n", myID, error_string, options->bayesmdimfilename);
+			    MPI_Error_string(my_file_open_error, error_string,
+					     &length_of_error_string);
+			    printf("%i> %s\n", myID, error_string);
+			    my_file_open_error = TRUE;
+			  }
+			MPI_File_set_size(world->mpi_bayesmdimfile, 0);		   
 #else
 			setup_filehandle_db((void *) world->bayesmdimfile, world, options,data);
 #endif
-		      }
+		      } //has bayesmdimfile
 #endif
 		  }			
 #ifdef MPI
@@ -804,11 +831,32 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 		  }
 		MYMPIBCAST (tempdb, maxfilenum, MPI_LONG, MASTER, comm_world);
 #endif
-    }
+}
     else
       {
 	// all non myID==MASTER
 #ifdef MPI
+#ifdef PARALIO
+	if(options->has_bayesmdimfile)
+	  {
+	    MPI_Info info;
+	    fprintf(stdout,"%i> before open %s\n",myID,options->bayesmdimfilename);
+	    my_file_open_error = MPI_File_open(MPI_COMM_SELF, options->bayesmdimfilename, 
+			  MPI_MODE_CREATE | MPI_MODE_WRONLY ,
+			  MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+	    if (my_file_open_error != MPI_SUCCESS) 
+	      {
+		MPI_Error_class(my_file_open_error, &error_class);
+		MPI_Error_string(error_class, error_string, &length_of_error_string);
+		printf("%i> @%s %s\n", myID, error_string, options->bayesmdimfilename);
+		MPI_Error_string(my_file_open_error, error_string,
+				 &length_of_error_string);
+		printf("%i> @%s\n", myID, error_string);
+		my_file_open_error = TRUE;
+	      }
+	    MPI_File_set_size(world->mpi_bayesmdimfile, 0);
+	  }
+#endif
 	MYMPIBCAST (tempdb, maxfilenum, MPI_LONG, MASTER, comm_world);
 	filenum = tempdb[0]; 
 	for(i=0, j=1;i<filenum;i++)
@@ -837,15 +885,17 @@ init_files (world_fmt * world, data_fmt * data, option_fmt * options)
 	      world->bayesfile = filedb[j++].file;
 	    // TEST: experiment with parallele I/O
 	    if(options->has_bayesmdimfile)
+	      {
 #ifdef PARALIO
-	      MPI_File_open(comm_world,options->bayesmdimfilename,MPI_MODE_CREATE,MPI_INFO_NULL, &world->mpi_bayesmdimfile);
+	      //has to be  dealt with before the broadcast	      MPI_File_open(comm_world,options->bayesmdimfilename,MPI_MODE_CREATE,MPI_INFO_NULL, &world->mpi_bayesmdimfile);
 #else
 #ifdef ZNZ
-	    world->bayesmdimfile = (znzFile) filedb[j++].file;
+		world->bayesmdimfile = (znzFile) filedb[j++].file;
 #else
-	      world->bayesmdimfile = filedb[j++].file;
+		world->bayesmdimfile = filedb[j++].file;
 #endif
 #endif
+	      }
 	  }
 #endif /*MPI*/
       }
