@@ -14,7 +14,7 @@
 Copyright 1996-2002 Peter Beerli and Joseph Felsenstein, Seattle WA
 Copyright 2003-2006 Peter Beerli, Tallahassee FL
  
-$Id: tools.h 1796 2011-01-25 08:33:50Z beerli $
+$Id: tools.h 1869 2011-05-06 12:08:40Z beerli $
 -------------------------------------------------------*/
 
 #include "migration.h"
@@ -101,7 +101,7 @@ extern void translate (char *text, char from, char to);
 extern void unpad (char *text, char removechars[]);
 extern void get_next_word(char **instring, char *delimiters, char **nextword);
 extern long count_words (char *text);
-extern void read_word(FILE *infile, char *word);
+extern int read_word(FILE *infile, char *word);
 extern void unread_word(FILE *infile, char *word);
 extern void add_to_buffer(char *fp, long *bufsize, char **buffer, long *allocbufsize);
 extern long print_to_buffer(char **buffer, long *maxbufsize, char *tempbuffer, long *pos, const char *fmt, ...);
