@@ -26,7 +26,7 @@ This software is distributed free of charge for non-commercial use
 and is copyrighted. Of course, we do not guarantee that the software
 works and are not responsible for any damage you may cause or have.
 
-$Id: world.h 1323 2008-07-25 19:13:48Z beerli $
+$Id: world.h 1515 2009-05-07 01:02:31Z beerli $
 -------------------------------------------------------*/
 
 #include "migration.h"
@@ -77,8 +77,6 @@ extern void clone_polish (world_fmt * original, world_fmt * kopie);
 extern long chance_swap_tree (world_fmt * tthis, world_fmt * that);
 extern void advance_clone_like (world_fmt * world, long accepted, long *j);
 extern void polish_world (world_fmt * world);
-extern void fill_worldoptions (worldoption_fmt * wopt, option_fmt * options);
-extern void fill_worlddata (worlddata_fmt * wdata, data_fmt * data);
 extern void print_alpha_curve (world_fmt * world, timearchive_fmt ** atl, long *gmaxptr);
 extern void print_cov (world_fmt * world, long numpop, long loci, MYREAL ***cov);
 extern void print_mighist (world_fmt * world);
